package stx.libjava.tools.compiler;

import java.io.PrintWriter;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import javax.tools.JavaCompiler;
import javax.tools.ToolProvider;

/**
 * Simple Java compiler adapter to compile classes completely in memory. 
 * @author Jan Vrany
 *
 */
public class JavaCompilerAdapter {
	protected List<String> options = Collections.emptyList();
	protected JavaCompiler compiler;
	protected JavaClassFileManager classFileManager;
	
	public JavaCompilerAdapter() {
		this(ToolProvider.getSystemJavaCompiler());
	}
	
	public JavaCompilerAdapter(JavaCompiler compiler) {
		this.setCompiler(compiler);
	}
	
	
	
	public void setCompiler(JavaCompiler compiler) {
		this.compiler = compiler;
		this.classFileManager = new JavaClassFileManager(compiler.getStandardFileManager(null, null, null));
	}
	
	public Collection<JavaClassFile> getClassFiles() {
		return classFileManager.getClasses().values();
	}
	
	/**
	 * Compiles classes in given source. The resulting .class files are added
	 * to an internal list which can be later retrieved by getClassFiles().
	 * 
	 * @param name unqualified name of class being compiled. 
	 * @param source source code of the class as String.   
	 * @return true, if compilation succeeded, false otherwise.
	 */
	
	public boolean compile(String name, String source) {
		List units = Arrays.asList(new JavaSourceFile(name, source));		
		JavaCompiler.CompilationTask task = compiler.getTask(new PrintWriter(System.err), classFileManager, null, options, null, units);
		return task.call();
	}

}
